# __init__.py
from joringels.src.actions.fetch import main as fetch
from joringels.src.actions.upload import main as upload
from joringels.src.actions.serve import main as serve
from joringels.src.actions.unprotectedload import main as unprotectedload
from joringels.src.actions.info import main as info
from joringels.src.actions.chkey import main as chkey
